SystemOrganization addCategory: #'Comet-Core'!
SystemOrganization addCategory: #'Comet-Examples'!
SystemOrganization addCategory: #'Comet-Platform'!

!Socket methodsFor: '*comet-squeak' stamp: 'lr 7/12/2006 14:57'!
pushData: aStringOrByteArray
	"Send aStringOrByteArray to the remote host trough the given socket. Push the data immediately and ignore all kinds of errors. Return true if all the data could be sent successfully, in case of problems return false and destroy the socket."

	| bytesSent bytesToSend count |
	bytesSent := 0.
	bytesToSend := aStringOrByteArray size.
	[ [ bytesSent < bytesToSend ] whileTrue: [
		count := self 
			primSocket: socketHandle
			sendData: aStringOrByteArray
			startIndex: bytesSent + 1
			count: (bytesToSend - bytesSent min: 5000).
		bytesSent := bytesSent + count.
		(count = 0 and: [ bytesSent ~= bytesToSend ])
			ifTrue: [ self destroy. ^ false ] ] ]
		on: Error do: [ self destroy. ^ false ].
	^ true! !

Object subclass: #CTHandler
	instanceVariableNames: 'id pusher socket session context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTHandler class methodsFor: 'instance creation' stamp: 'lr 8/30/2006 11:56'!
on: aPusher session: aSession context: aContext
	^ self basicNew initializeOn: aPusher session: aSession context: aContext! !

!CTHandler methodsFor: 'private' stamp: 'lr 8/30/2006 12:09'!
basicConnect: aRequest
	| root |
	socket := aRequest responseStream socket.
	socket pushData: (String streamContents: [ :stream |
		root := WARenderedHtmlRoot context: context.
		WAResponse new
			contentType: 'text/html; charset=utf-8';
			headerAt: 'Expires' put: 'Thu, 11 Jun 1980 12:00:00 GMT';
			headerAt: 'Cache-Control' put: 'no-cache, must-revalidate';
			headerAt: 'Pragma' put: 'no-cache';
			contents: (String streamContents: [ :contents |
				WACurrentSession
					use: session
					during: [ session updateRoot: root. root writeOn: contents ].
				contents nextPutAll: (String new: 1024 withAll: Character space) ]);
			writeOn: stream ])! !

!CTHandler methodsFor: 'private' stamp: 'lr 7/12/2006 14:33'!
basicDisconnect
	socket ifNil: [ ^ self ].
	socket destroy.
	socket := nil! !

!CTHandler methodsFor: 'private' stamp: 'lr 7/12/2006 13:56'!
basicTerminate
	thisContext swapSender: nil.
	Processor activeProcess terminate
	! !

!CTHandler methodsFor: 'actions' stamp: 'lr 8/30/2006 11:32'!
connect: aRequest
	self 
		basicDisconnect; 
		basicConnect: aRequest.
	self pusher register: self.
	self basicTerminate! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:22'!
id
	^ id! !

!CTHandler methodsFor: 'initialization' stamp: 'lr 8/30/2006 11:56'!
initializeOn: aPusher session: aSession context: aContext
	id := WAExternalID new asString.
	pusher := aPusher.
	session := aSession.
	context := aContext! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 14:24'!
pusher
	^ pusher! !

!CTHandler methodsFor: 'actions' stamp: 'lr 8/30/2006 12:11'!
push: aString
	socket ifNil: [ ^ false ].
	^ socket pushData: aString! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 8/30/2006 11:32'!
session
	^ session! !

Object subclass: #CTPusher
	instanceVariableNames: 'mutex handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTPusher class
	instanceVariableNames: 'process delay'!

!CTPusher class methodsFor: 'initialization' stamp: 'mb 10/4/2006 23:19'!
initialize
	SeasidePlatformSupport 
		addToStartUpList: self after: WAListener; 
		addToShutDownList: self.
	self startUp! !

!CTPusher class methodsFor: 'instance creation' stamp: 'mb 10/5/2006 14:55'!
new
	^self basicNew initialize! !

!CTPusher class methodsFor: 'private' stamp: 'lr 8/30/2006 10:25'!
pingProcess
	self allInstancesDo: [ :each | each ping ].
	delay wait! !

!CTPusher class methodsFor: 'initialization' stamp: 'mb 10/5/2006 11:48'!
shutDown
	process ifNil: [ ^ self ].
	SeasidePlatformSupport terminateProcess: process.
	process := nil! !

!CTPusher class methodsFor: 'initialization' stamp: 'mb 10/5/2006 12:45'!
startUp
	self shutDown.
	delay := Delay forSeconds: 15.
	process := SeasidePlatformSupport
		withName: 'ping'
		withLowestPriorityFork: [ [ self pingProcess ] repeat ]
! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:50'!
handlers
	"Answer the current list of handlers."

	^ handlers! !

!CTPusher methodsFor: 'initialization' stamp: 'mb 10/5/2006 14:57'!
initialize
	mutex := Semaphore forMutualExclusion.
	handlers := Array new! !

!CTPusher methodsFor: 'public' stamp: 'mb 10/5/2006 23:15'!
javascript: anObject
	"Push aString to all associated handlers."

	self push: (String streamContents: [ :stream |
		stream 
			nextPutAll: '<script type="text/javascript">parent.Comet.eval(';
			javascript: anObject displayString;
			nextPutAll: ')</script>' ])! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 15:01'!
mutex
	"Answer the mutex in use with the reciever."

	^ mutex! !

!CTPusher methodsFor: 'public' stamp: 'lr 8/30/2006 11:15'!
ping
	"Ping all the associated handlers."

	self mutex critical: [ handlers := handlers select: [ :each | each push: ' ' ] ]! !

!CTPusher methodsFor: 'public' stamp: 'lr 8/30/2006 11:14'!
push: aString
	"Push aString to all associated handlers."

	handlers do: [ :each | each push: aString ]! !

!CTPusher methodsFor: 'public' stamp: 'lr 8/30/2006 10:43'!
register: aHandler
	"Register a new handler into this pusher."

	mutex critical: [ handlers := handlers copyWith: aHandler ]! !

!WARenderCanvas methodsFor: '*comet' stamp: 'lr 7/12/2006 12:17'!
comet
	^ CTComet on: self! !

SUFunctionalTest subclass: #CTChat
	instanceVariableNames: 'username message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

CTChat class
	instanceVariableNames: 'pusher messageClass'!

!CTChat class methodsFor: 'initialization' stamp: 'mb 10/10/2006 19:52'!
initialize
	| application |
	pusher := CTPusher new.
	application := self registerAsApplication: 'comet/chat'.
	application preferenceAt: #sessionClass put: CTChatSession.
	application libraries add: SULibrary; add: CTLibrary! !

!CTChat class methodsFor: 'accessing' stamp: 'mb 10/10/2006 19:31'!
messageClass
	| result |
	messageClass isNil ifTrue: [messageClass := 'odd'].
	result := messageClass.
	messageClass := messageClass = 'odd'
		ifTrue: ['even']
		ifFalse: ['odd'].
	^result.! !

!CTChat class methodsFor: 'accessing' stamp: 'lr 7/12/2006 15:04'!
pusher
	^ pusher! !

!CTChat methodsFor: 'private' stamp: 'mb 10/10/2006 20:02'!
event: aString
	self pusher javascript: (SUInsertion bottom
		id: 'messages';
		with: (self htmlStringClass: 'event' with: aString))! !

!CTChat methodsFor: 'private' stamp: 'mb 10/10/2006 20:20'!
htmlChatMembers
	| aStream aString |
	aStream := String new writeStream.
	self otherHandlers do:
		[:h |
		aStream nextPutAll: h session username]
	separatedBy:
		[aStream nextPutAll: ', '].
	aString := aStream contents.
	aString := aString isEmpty
		ifTrue: ['You are the only one there']
		ifFalse:	['Also there: ' , aStream contents].
	^ 
'<div class="message message-' , self class messageClass , '">' , 
	'<span class="message">' , aString , '</span>' ,
'</div>'! !

!CTChat methodsFor: 'private' stamp: 'mb 10/10/2006 20:01'!
htmlStringClass: aCssClass with: aString
	^ 
'<div class="message message-' , self class messageClass , '">' , 
	'<span class="username">' , self username , '</span>' ,
	'<span class="' , aCssClass , '">' , aString , '</span>' ,
'</div>'! !

!CTChat methodsFor: 'testing' stamp: 'mb 10/10/2006 19:55'!
isLoggedIn
	^self username notNil! !

!CTChat methodsFor: 'private' stamp: 'mb 10/10/2006 20:02'!
message: aString
	self pusher javascript: (SUInsertion bottom
		id: 'messages';
		with: (self htmlStringClass: 'message' with: aString))! !

!CTChat methodsFor: 'private' stamp: 'mb 10/10/2006 20:18'!
otherHandlers
	^self pusher handlers reject:
		[:h | h session == self session 
			or: [h session username isNil]]! !

!CTChat methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:40'!
pusher
	^ self class pusher! !

!CTChat methodsFor: 'rendering' stamp: 'mb 10/10/2006 20:05'!
renderChatOn: html
	html div id: 'status'; with: [
		html text: 'Currently logged in as ', self username , '.'.
		html space.
		html anchor
			callback: [self event: 'is saying bye, bye...'. self username: nil];
			text: 'Logout'
		].		 
	html div id: 'messages'.
	html textArea
		id: 'message';
		callback: [ :value | self message: value ];
		onEnter: (html request
			triggerFormElement: 'message';
			onSuccess: (SUStream on: '$("message").value = ""')).
	html script: (html comet
		pusher: self pusher;
		connect).
	html script: (SUInsertion bottom
		id: 'messages';
		with: self htmlChatMembers)
! !

!CTChat methodsFor: 'rendering' stamp: 'mb 10/10/2006 19:37'!
renderContentOn: html
	html div class: 'chat'; with: [
		html heading: 'Seaside Chat' level: 3.
		self isLoggedIn
			ifFalse:
				[self renderLoginOn: html]
			ifTrue:
				[self renderChatOn: html]
		]! !

!CTChat methodsFor: 'rendering' stamp: 'mb 10/10/2006 19:54'!
renderLoginOn: html
	html form: [
		html textInput
			id: 'username';
			value: self username;
			callback: [ :value | self username: value. value notNil ifTrue: [self event: 'joined' ] ].
		html submitButton
			text: 'Login'.
	]
	
! !

!CTChat methodsFor: 'rendering' stamp: 'mb 10/10/2006 19:34'!
style
	^ '
div.chat {
	font-family: Tahoma;
}
div.message {
	margin-top: 3px;
	margin-bottom: 3px;
}
div.message-odd {
	background-color: #dddddd;
}
div.message-even {
	background-color: #eeeeee;
}
span.username {
	margin-right: 1em;
	font-weight: bold;
}
span.event {
	font-style: italic;
}
#message {
	width: 80%;
	height: 5em; 
	margin: 1em;
}
#messages {
	height: 16em;
	overflow: auto;
}
#status {
	width: 100%;
	padding: 2em;
	background-color: #lightblue;
}

' 
! !

!CTChat methodsFor: 'accessing' stamp: 'mb 10/10/2006 19:53'!
username
	^self session username! !

!CTChat methodsFor: 'accessing' stamp: 'mb 10/10/2006 19:53'!
username: aString
	self session username: aString! !

SUFunctionalTest subclass: #CTCounter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

CTCounter class
	instanceVariableNames: 'pusher count'!

!CTCounter class methodsFor: 'private' stamp: 'lr 8/30/2006 12:28'!
count
	^ count! !

!CTCounter class methodsFor: 'initialization' stamp: 'lr 8/30/2006 12:27'!
initialize
	| application |
	pusher := CTPusher new.
	count := ValueHolder new contents: 0.
	count addDependent: self.
	application := self registerAsApplication: 'comet/counter'.
	application preferenceAt: #sessionClass put: CTSession.
	application libraries add: SULibrary; add: CTLibrary! !

!CTCounter class methodsFor: 'private' stamp: 'lr 8/30/2006 12:30'!
pusher
	^ pusher! !

!CTCounter class methodsFor: 'private' stamp: 'lr 8/30/2006 13:54'!
update: anAspect
	pusher javascript: (SUElement new
		id: 'count';
		update: self count contents asString)! !

!CTCounter methodsFor: 'accessing' stamp: 'lr 8/30/2006 12:27'!
count
	^ self class count contents! !

!CTCounter methodsFor: 'accessing' stamp: 'lr 8/30/2006 12:27'!
count: anInteger
	^ self class count contents: anInteger! !

!CTCounter methodsFor: 'accessing' stamp: 'lr 8/30/2006 12:30'!
pusher
	^ self class pusher! !

!CTCounter methodsFor: 'rendering' stamp: 'lr 8/30/2006 13:55'!
renderContentOn: html
	html heading
		id: 'count';
		with: self count.
	html anchor
		onClick: (html request 
			callback: [ self count: self count + 1 ]);
		with: '++'.
	html space.
	html anchor
		onClick: (html request 
			callback: [ self count: self count - 1 ]);
		with: '--'.
	html script: (html comet
		pusher: self pusher;
		connect)! !

!WriteStream methodsFor: '*comet-squeak' stamp: 'lr 7/3/2006 14:14'!
crlf
	self nextPutAll: String crlf! !

WASession subclass: #CTSession
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTSession subclass: #CTChatSession
	instanceVariableNames: 'username'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Examples'!

!CTChatSession methodsFor: 'accessing' stamp: 'mb 10/10/2006 19:51'!
username
	"Answer the value of username"

	^ username! !

!CTChatSession methodsFor: 'accessing' stamp: 'mb 10/10/2006 19:51'!
username: anObject
	"Set the value of username"

	username _ anObject! !

!CTSession methodsFor: 'processing' stamp: 'lr 8/30/2006 12:04'!
cometRequest: aRequest
	^ (handlers
		at: (aRequest fields
			at: CTComet headerField
			ifAbsent: [ ^ nil ])
		ifAbsent: [ ^ nil ])
			connect: aRequest! !

!CTSession methodsFor: 'processing' stamp: 'lr 7/12/2006 12:10'!
incomingRequest: aRequest
	aRequest responseStream
		ifNil: [ self error: 'Streaming-server required.' ].
	^ (aRequest fields includesKey: CTComet headerField)
		ifTrue: [ self cometRequest: aRequest ]
		ifFalse: [ super incomingRequest: aRequest ]! !

!CTSession methodsFor: 'initialization' stamp: 'lr 7/12/2006 11:02'!
initializeWithApplication: anApplication
	super initializeWithApplication: anApplication.
	handlers := Dictionary new! !

!CTSession methodsFor: 'public' stamp: 'lr 8/30/2006 11:57'!
registerPusher: aPusher context: aContext
	| handler |
	handler := CTHandler on: aPusher session: self context: aContext.
	handlers at: handler id put: handler.
	^ handler id! !

WAScriptLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTLibrary methodsFor: 'scripts' stamp: 'lr 8/30/2006 14:23'!
comet
	^ '/* Seaside Comet API
 *  (c) 2006 Lukas Renggli (renggli@iam.unibe.ch)
 */

Comet = Class.create();
Comet.prototype = {
	initialize: function (url, options) {
		this.url = url;
		this.options = Object.extend({
			reconnect: false
		}, options || {});
		this.iframe = document.createElement("iframe");
		this.iframe.style.visibility = "hidden";
		this.iframe.style.width = this.iframe.style.height = this.iframe.style.border = "0";
		document.getElementsByTagName("body")[0].appendChild(this.iframe);
	},
	connect: function() {
		this.iframe.onLoad = this.onLoad.bind(this);
		this.iframe.src = this.url;
	},
	disconnect: function() {
		this.iframe.onLoad = Prototype.emptyFunction; 
		this.iframe.src = "about:blank";
	},
	onLoad: function() {
		this.disconnect();
		if (this.options.reconnect)
			this.connect();
	}
};
Comet.eval = function(code) { return eval(code); };'! !

SUPrototype subclass: #CTComet
	instanceVariableNames: 'pusher url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/12/2006 12:10'!
headerField
	^ '_c'! !

!CTComet class methodsFor: 'instance-creation' stamp: 'mb 10/5/2006 14:54'!
new
	^self basicNew initialize! !

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/12/2006 12:10'!
terminator
	^ Character value: 0! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
connect
	self call: 'connect'! !

!CTComet methodsFor: 'private' stamp: 'lr 5/28/2006 22:00'!
defaultArguments
	^ (Array with: self url) , super defaultArguments! !

!CTComet methodsFor: 'private' stamp: 'lr 5/28/2006 22:07'!
defaultPrototype
	^ 'Comet'! !

!CTComet methodsFor: 'private' stamp: 'lr 7/12/2006 12:10'!
defaultUrl
	^ self canvas context actionUrl
		addParameter: self class headerField
		value: self pusher id! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
disconnect
	self call: 'disconnect'! !

!CTComet methodsFor: 'initialization' stamp: 'lr 7/3/2006 13:33'!
initialize
	super initialize.
	self create! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:44'!
pusher
	^ pusher! !

!CTComet methodsFor: 'accessing' stamp: 'lr 8/30/2006 11:57'!
pusher: aPusher
	self url: (self canvas context actionUrl
		withParameter: self class headerField 
		value: (self session 
			registerPusher: (pusher := aPusher)
			context: self canvas context))! !

!CTComet methodsFor: 'options' stamp: 'lr 7/3/2006 13:37'!
reconnect: aBoolean
	self optionAt: 'reconnect' put: aBoolean! !

!CTComet methodsFor: 'initialization' stamp: 'lr 7/3/2006 13:32'!
setPusher: aPusher
	pusher := aPusher! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:33'!
url
	^ url ifNil: [ url := self defaultUrl ]! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:34'!
url: anUrl
	url := anUrl! !

SeasideAddonVWInfo subclass: #CometVWInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Platform'!

!CometVWInfo class methodsFor: 'actions' stamp: 'mb 10/4/2006 22:59'!
fileOutVW5
	(Smalltalk classNamed: #VW5PackageExporter) ifNotNilDo: [ :class |
		class
			fileOutPackage: self default 
			on: (FileStream forceNewFileNamed: 'CometVW5.st') ]! !

!CometVWInfo methodsFor: 'accessing' stamp: 'mb 10/7/2006 18:00'!
addonName
	^'Comet'! !

!SeasidePlatformSupport class methodsFor: '*comet-squeak' stamp: 'mb 10/4/2006 23:18'!
addToShutDownList: anObject
	Smalltalk addToShutDownList: anObject.! !

!SeasidePlatformSupport class methodsFor: '*comet-squeak' stamp: 'mb 10/4/2006 23:18'!
addToStartUpList: anObject after: another
	Smalltalk addToStartUpList: anObject after: another 
! !

!SeasidePlatformSupport class methodsFor: '*comet-squeak' stamp: 'mb 10/5/2006 11:47'!
terminateProcess: aProcess
	^aProcess terminate! !

!SeasidePlatformSupport class methodsFor: '*comet-squeak' stamp: 'mb 10/5/2006 12:44'!
withName: aString withLowestPriorityFork: aBlock
	^self
		withName: aString
		withPriority: Processor lowestPriority
		fork: aBlock! !

!SeasidePlatformSupport class methodsFor: '*comet-squeak' stamp: 'mb 10/5/2006 12:43'!
withName: aString withPriority: anInteger fork: aBlock 
	^aBlock forkAt: anInteger named: aString! !
CTPusher initialize!
CTChat initialize!
CTCounter initialize!
